PASS_RESET = 'PASS_RESET'
PASS_RESET_ACCESS = 'PASS_RESET_ACCESS'
TEMPLATE_PATHS = 'JWT_ALLAUTH_TEMPLATES'

EMAIL_VERIFIED_REDIRECT = 'EMAIL_VERIFIED_REDIRECT'
PASSWORD_RESET_REDIRECT = 'PASSWORD_RESET_REDIRECT'

PASS_RESET_COOKIE = 'password_reset_access_token'

FOR_USER = 'for_user'
ONE_TIME_PERMISSION = 'one_time_permission'

REFRESH_TOKEN_COOKIE = 'refresh_token'

# Admin-managed registration & email confirmation flow
PASS_SET = 'PASS_SET'
PASS_SET_ACCESS = 'PASS_SET_ACCESS'
PASSWORD_SET_REDIRECT = 'PASSWORD_SET_REDIRECT'
SET_PASSWORD_COOKIE = 'set_password_access_token'
EMAIL_CONFIRMATION = 'EMAIL_CONFIRMATION'

# MFA
MFA_SALT = 'jwt_allauth_mfa'
MFA_TOKEN_MAX_AGE_SECONDS = 300

# MFA TOTP modes
MFA_TOTP_DISABLED = 'disabled'
MFA_TOTP_OPTIONAL = 'optional'
MFA_TOTP_REQUIRED = 'required'
